<h1>
    <!--% ordinary_default %-->
</h1>
<div>
    <!--% config.default1 %-->
</div>
<h4><!--% non_config_var %--></h4>
<span>
    <!--% config.default2 %-->
</span>
<h2>
    <!--% config.nested.iexist %-->
</h2>
<h3>
    <!--% config.nested.idontexist %-->
</h3>
